"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeValueForBidUsingLatestBlockTime = exports.computeValueForBid = exports.percentage = exports.minutes = exports.seconds = void 0;
__exportStar(require("./generated"), exports);
__exportStar(require("./hooked"), exports);
__exportStar(require("./constants"), exports);
__exportStar(require("./create"), exports);
__exportStar(require("./flattenNodes"), exports);
__exportStar(require("./plugin"), exports);
// Floting point precision for percentages.
const DECIMAL = 2;
// Helper to convert a number represeting seconds to a bigint.
const seconds = (x) => BigInt(x);
exports.seconds = seconds;
// Helper to convert a number represeting minutes to a bigint.
const minutes = (x) => BigInt(x) * 60n;
exports.minutes = minutes;
// Helper to convert the percentage vaue to a number.
const percentage = (x) => {
    const precision = 10 ** DECIMAL;
    const value = Math.floor(x * precision);
    const remaining = x - value / precision;
    if (x > precision) {
        throw new Error(`value must be between 0 and ${precision}, got ${x}`);
    }
    else if (remaining > 0) {
        throw new Error(`value must have only ${DECIMAL} decimal precision, got ${x}`);
    }
    return Number(value);
};
exports.percentage = percentage;
// Helper to determine the adjusted value of a 'Dutch' auction.
function computeValueForBid(context, auction, currentTime) {
    const { startValue, minimumValue, startDate, endDate } = auction;
    const currentTimeDelta = Number(endDate - currentTime);
    const timeDelta = Number(endDate - startDate);
    const priceSegment = Number(startValue - minimumValue);
    const decay = currentTimeDelta / timeDelta;
    const decayedPricePoint = decay * priceSegment;
    return BigInt(Math.floor(decayedPricePoint)) + minimumValue;
}
exports.computeValueForBid = computeValueForBid;
// Helper to determine the adjusted value of a 'Dutch' auction
// using the latest block time.
async function computeValueForBidUsingLatestBlockTime(context, auction) {
    const currentSlot = await context.rpc.getSlot();
    const currentTime = await context.rpc.getBlockTime(currentSlot);
    if (!currentTime) {
        throw new Error('Unable to get current cluster time');
    }
    return computeValueForBid(context, auction, currentTime);
}
exports.computeValueForBidUsingLatestBlockTime = computeValueForBidUsingLatestBlockTime;
//# sourceMappingURL=index.js.map